open Lwt.Infix

let to_cohttp_req req =
  let http_method =
    req |> Request.get_http_method |> Cohttp.Code.method_of_string in
  let uri = Request.get_uri req in
  let headers =
    req |> Request.get_headers |> Headers.to_list |> Cohttp.Header.of_list in
  let body =
    req |> Request.get_body |> Body.to_stream |> Cohttp_lwt.Body.of_stream in
  (http_method, uri, headers, body)

let to_okweb_resp cresp body =
  let status =
    cresp |> Cohttp.Response.status |> Cohttp.Code.code_of_status in
  let headers =
    cresp |> Cohttp.Response.headers |> Cohttp.Header.to_list |> Headers.of_list in
  Response.({ status = status; headers = headers; body = body }) |> Lwt.return

let send req =
  let (http_method, uri, headers, body) = to_cohttp_req req in
  Cohttp_lwt_unix.Client.call ~headers ~body http_method uri >>= fun (cresp, cbody) ->
  begin match Cohttp_lwt.Body.transfer_encoding cbody with
    | Cohttp.Transfer.Fixed _ ->
      Cohttp_lwt.Body.to_string cbody |> Lwt.map Body.of_string
    | Cohttp.Transfer.Chunked
    | Cohttp.Transfer.Unknown ->
      Cohttp_lwt.Body.to_stream cbody |> Body.of_stream |> Lwt.return
  end >>= fun body ->
  to_okweb_resp cresp body
