module Same_site = struct
  type t = Strict | Lax | None [@@deriving show { with_path = false }, eq]
end

type t = {
  key: string;
  value: string;
  expires: Core.Time.t option;
  max_age: int64 option;
  domain: string option;
  path: string option;
  secure: bool;
  http_only: bool;
  same_site: Same_site.t option;
} [@@deriving show, eq]

let make
    ?expires
    ?max_age
    ?domain
    ?path
    ?(secure=false)
    ?(http_only=false)
    ?same_site
    (key, value) =
  { key; value; expires; max_age; domain; path; secure; http_only; same_site }

let to_pair { key; value; _ } =
  (key, value)

let get_key { key; _ } =
  key

let get_value { value; _ } =
  value

let get_expires { expires; _ } =
  expires

let get_max_age { max_age; _ } =
  max_age

let get_domain { domain; _ } =
  domain

let get_path { path; _ } =
  path

let is_secure { secure; _ } =
  secure

let is_http_only { http_only; _ } =
  http_only

let get_same_site { same_site; _ } =
  same_site

let set_key key c =
  { c with key = key }

let set_value value c =
  { c with value = value }

let set_expires expires c =
  { c with expires = expires }

let set_max_age max_age c =
  { c with max_age = max_age }

let set_domain domain c =
  { c with domain = domain }

let set_path path c =
  { c with path = path }

let set_secure secure c =
  { c with secure = secure }

let set_http_only http_only c =
  { c with http_only = http_only }

let set_same_site same_site c =
  { c with same_site = same_site }
