open Base
open Lwt.Infix

let log_request_response log req resp =
  let remote_host = match req |> Request.get_client_addr with
    | Some (Unix.ADDR_INET (inet_addr, _)) ->
      Core.Unix.Inet_addr.to_string inet_addr
    | Some (Unix.ADDR_UNIX pipe) ->
      pipe
    | None ->
      "<unknown host>"
  in
  let zone = Lazy.force Core.Time.Zone.local in
  let req_end_time = Core.Time.(now () |> to_string_iso8601_basic ~zone) in
  let http_method = Request.get_http_method req in
  let path = req |> Request.get_uri |> Uri.path_and_query in
  let resp_code = Response.get_status resp in
  let content_length = resp |> Response.get_body |> Body.length |> Option.value ~default:(-1) in
  log (Printf.sprintf "%s - - [%s] \"%s %s\" %d %d" remote_host req_end_time http_method path resp_code content_length)

let middleware ~log handler req =
  Lwt.catch
    (fun () -> handler req >|= fun resp ->
      log_request_response log req resp;
      resp)
    (fun ex ->
       log_request_response log req Response.(of_status 500);
       Lwt.fail ex)
