open Base

type t = {
  status: int;
  headers: Headers.t;
  body: Body.t;
} [@@deriving show]

let get_status {status; _} =
  status

let get_headers {headers; _} =
  headers

let get_body {body; _} =
  body

let get_cookies {headers; _} =
  headers
  |> Headers.find_all ~key:"Set-Cookie"
  |> Cookie_jar.of_set_cookie_headers

let find_header resp ~key =
  Headers.find resp.headers ~key

let ok =
  { status = 200; headers = []; body = Body.empty; }

let of_status status =
  { ok with status = status }

let with_headers headers resp =
  { resp with headers = headers }

let with_header ~key ~value resp =
  with_headers (Headers.set ~key ~value resp.headers) resp

let with_body body resp =
  let new_headers =
    get_headers resp
    |> Body.set_encoding_and_length_headers body in
  { resp with body = body; headers = new_headers }

let with_string_body body resp =
  with_body (Body.of_string body) resp

let with_cookies jar resp =
  let set_cookie_values = Cookie_jar.to_set_cookie_headers jar in
  let new_headers = Headers.set_all resp.headers ~key:"Set-Cookie" ~values:set_cookie_values in
  with_headers new_headers resp

let redirect ?(status=303) url =
  of_status status
  |> with_header ~key:"Location" ~value:url
  |> with_header ~key:"Content-Type" ~value:"text/html; charset=utf-8"
  |> with_string_body (Printf.sprintf "<a href=\"%s\">Moved Permanently</a>" url)
