open Alcotest
open Base
open Okweb

let test_search_query_params =
  test_case "search query params" `Quick begin fun () ->
    let uri = "?foo=one,two,three&bar=four" in
    let req = Request.get uri in
    check (option (list string))
      "searching all query values"
      (Some ["one"; "two"; "three"])
      (Request.find_query_params req ~key:"foo");
    check (option string)
      "searching the 1st query value for key 'bar'"
      (Some "four")
      (Request.find_query_param req ~key:"bar");
    check (option string)
      "searching the 1st query value for key 'foo'"
      (Some "one")
      (Request.find_query_param req ~key:"foo");
  end

let test_path =
  test_case "path parsing" `Quick begin fun () ->
    let cases = [
      "", [];
      "/", [];
      "/one", ["one"];
      "/one/two", ["one"; "two"];
      "/one/two/three", ["one"; "two"; "three"];
      "/one//two///three//", ["one"; "two"; "three"];
    ] in
    List.iter cases ~f:begin fun (input, output) ->
      check (list string)
        (Printf.sprintf "checking parsed value of path '%s'" input)
        output
        (Request.get input |> Request.get_path)
    end
  end

let test_search_header =
  test_case "search header" `Quick begin fun () ->
    let value = "Test Value" in
    let test_headers = Headers.of_list ["test-header", value] in
    let req = Request.(get "/" |> with_headers test_headers) in
    let cases = [
      "test-header";
      "TEST-HEADER";
      "test-HEADER";
      "tEsT-hEaDeR";
    ] in
    List.iter cases ~f:begin fun key ->
      check (option string)
        (Printf.sprintf "searching value of header '%s'" key)
        (Some value)
        (Request.find_header req ~key)
    end
  end

let cases = [
  test_search_query_params;
  test_path;
  test_search_header;
]
