open Alcotest
open Base
open Okweb
open Lwt.Infix

let make_random_string () =
  let random_printable_ascii () = (32 + Random.int 94) |> Char.of_int_exn in
  String.init 16 ~f:(fun _ -> random_printable_ascii ())

let test_smoke_test_of_server_and_client =
  test_case "smoke test" `Quick begin fun () ->
    let handler req =
      let header_value =
        req
        |> Request.find_header ~key:"X-Test-Request"
        |> Option.value ~default:"INVALID" in
      let cookie_value =
        req
        |> Request.get_cookies
        |> Cookie_jar.find_value ~key:"test-request"
        |> Option.value ~default:"INVALID" in
      (req |> Request.get_body |> Body.read_up_to) >>= fun body ->
      Lwt.return Response.(
          ok
          |> with_header ~key:"X-Test-Response" ~value:header_value
          |> with_cookies Cookie_jar.(empty |> set Cookie.(make ("test-response", cookie_value)))
          |> with_string_body body)
    in

    let port = 54321 in
    let uri = Printf.sprintf "http://127.0.0.1:%d/" port in
    let test_header_value = make_random_string () in
    let test_cookie_value = make_random_string () in
    let req = Request.(
        post uri
        |> with_header ~key:"X-Test-Request" ~value:test_header_value
        |> with_cookies Cookie_jar.(empty |> set Cookie.(make ("test-request", test_cookie_value)))
        |> with_string_body "OK") in
    let (resp, body) = Lwt_main.run begin
        Lwt.async (fun () -> Server.listen ~address:"127.0.0.1" ~port handler);
        Client.send req >>= fun resp ->
        Body.read_up_to (Response.get_body resp) >>= fun body ->
        Lwt.return (resp, body)
      end in

    check (option string)
      "checking test header"
      (Some test_header_value)
      (Response.find_header ~key:"X-Test-Response" resp);
    check string
      "checking body"
      "OK"
      body;
    check (option string)
      "checking test cookie"
      (Some test_cookie_value)
      (Response.get_cookies resp |> Cookie_jar.find_value ~key:"test-response");
  end


let cases = [
  test_smoke_test_of_server_and_client;
]
